<?php
namespace SupplyTek\Controllers;

use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use SupplyTek\Utilities\CrudSupplyTek;
use SupplyTek\Utilities\SessionHandler;
use Doctrine\ORM\EntityManager;

/**
 * Handles Products related requests
 * 
 * Controls received requests related to products, either
 * products that belongs to a category or a specific product.
 *
 * @author Rômulo Alves Lousada
 */
class ProductController 
{
    protected $crud;
    protected $sessionHandler;
    protected $fileServerURL;
    protected $connection;

    /**
     * Class constructor
     * 
     * Initialize an object's properties upon creation of the object.
     * 
     * @param object $crud Object that manipulates database queries
     * @param object $sessionHandler Object that allows to manipulate PHP Session
     * 
     * @return void
     * 
     * @access public
     * @author Rômulo Alves Lousada
     */
    public function __construct ($crud, $sessionHandler, $fileServerURL, EntityManager $entityManager) 
    {
        $this->crud = $crud;
        $this->sessionHandler = $sessionHandler;
        $this->fileServerURL = $fileServerURL;
        $this->connection = $entityManager->getConnection();
    }

    /**
     * Return all products from catalog
     * 
     * Receives data from request and filters all products from catalog.
     * 
     * @param Request $request Object that receives data from the caller
     * @param Response $response Object that sends back data to the caller
     *
     * @return array
     * @throws Exception
     * 
     * @access public
     * @author Rômulo Alves Lousada
     */
    public function getProductsCatalog(Request $request, Response $response)
    {
        try {
            $filterUri = $request->getParameters();
            $filterCriteria = $request->getFilterCriteria();
            $queryParams = array(
                'ID' => 'T',
                'CATEGORY_ID' => 'T',
                'SUBCATEGORY_ID' => 'T',
                'SUPPLIER_ID' => 'T',
                'BRAND_ID' => 'T',
                "BRANCH_ID" => $this->sessionHandler->getSessionData("CDFILIAL")
            );
            
            $filter = $this->crud->processFilter($filterCriteria, $filterUri);
            
            $params = array(
                "OPERATOR_ID" => $this->sessionHandler->getSessionData("CDOPERADOR")
            );
            foreach ($queryParams as $column=>$value) {
                if (!isset($filter[$column])) {
                    $params[$column] = $value;
                };
            }
            $products = $this->crud->find($request, $params);
        
            $newDataSet = $this->setImgFullPath($products);
            
            $response->setMetadata($newDataSet);
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    /**
     * Returns the most purchased products
     * 
     * Receive data and filter the most purchased products in the last three 
     * months by the unit logged into the system.
     * 
     * @param Request $request Object that receives data from the caller
     * @param Response $response Object that sends back data to the caller
     *
     * @return array
     * 
     * @access public
     * @author Bruna Chaves Carneiro
     */
    public function getMostPurchasedProducts(Request $request, Response $response)
    {
        try {
            $params = array(
                "P_CDFILIAL" => $this->sessionHandler->getSessionData("CDFILIAL")
            );
            
            $products = $this->crud->find($request, $params);
            $newDataSet = $this->setImgFullPath($products);
            
            $response->setMetadata($newDataSet);
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    /**
     * Returns full image address
     * 
     * Get the product image link on the file server and link it with the image 
     * link brought from the bank so that the image is displayed on the system.
     *
     * @param string $products – Array with catalog products
     * 
     * @return array
     * 
     * @acess private
     * @author Bruna Chaves Carneiro
     */
	private function setImgFullPath($products) 
	{
        if(!empty($products['data'])) {
    	    if (array_key_exists(0, $products['data'])) {
        		foreach ($products['data'] as &$product) {
        		    $image = $product['IMAGE'] == null ? true : false;
        		    $description = $product['DESCRIPTION'] == null ? true : false;
        		    if($image || $description) {
        		        $sql = "SELECT PS.DSPATHFOTO, PS.DSDESCRICAO FROM PRODSUPPLYTEK PS INNER JOIN PRODUTO P ON PS.CDPRODUTO = P.CDPRODUTO AND P.CDARVPROD = :P_CDPRODUTO";
                        $params = array(
                            "P_CDPRODUTO" => $product['CODE']
                        );
                
                        $std = $this->connection->prepare($sql);
                        $std->execute($params);
                        $result = $std->fetch();
                        if($image && $description) {
                            $product['IMAGE'] = $result['DSPATHFOTO'];
                            $product['DESCRIPTION'] = $result['DSDESCRICAO'];
                        }else if($image) {
        		            $product['IMAGE'] = $result['DSPATHFOTO'];
            		    } else {
            		        $product['DESCRIPTION'] = $result['DSDESCRICAO'];
            		    }
        		    }
        			if ($product['IMAGE']) {
        				$product['IMAGE'] = $this->fileServerURL . $product['IMAGE'];
        			} 
        		}
            } else {
                $image = $products['data']['IMAGE'] == null ? true : false;
    		    $description = $products['data']['DESCRIPTION'] == null ? true : false;
    		    if($image || $description) {
                    $sql = "SELECT PS.DSPATHFOTO, PS.DSDESCRICAO FROM PRODSUPPLYTEK PS INNER JOIN PRODUTO P ON PS.CDPRODUTO = P.CDPRODUTO AND P.CDARVPROD = :P_CDPRODUTO";
            
                    $params = array(
                        "P_CDPRODUTO" => $products['data']['CODE']
                    );
            
                    $std = $this->connection->prepare($sql);
                    $std->execute($params);
                    $result = $std->fetch();
                    if($image && $description) {
                        $products['data']['IMAGE'] = $result['DSPATHFOTO'];
                        $products['data']['DESCRIPTION'] = $result['DSDESCRICAO'];
                    }else if($image) {
    		            $products['data']['IMAGE'] = $result['DSPATHFOTO'];
        		    } else {
        		        $products['data']['DESCRIPTION'] = $result['DSDESCRICAO'];
        		    }
    		    } 
                if($products['data']['IMAGE']) {
                    $products['data']['IMAGE'] = $this->fileServerURL . $products['data']['IMAGE'];
                }
            }
        }
		return $products;
	}
	
	/**
     * Receives parameters from the front end, executes the query to bring data 
     * from the product filter and returns this data to the front end
     * 
     * @param Request $request Object that receives data from the caller
     * @param Response $response Object that sends back data to the caller
     *
     * @return array
     * @throws Exception
     * 
     * @access public
     * @author Grazielle Ferreira Antunes 
     */
    public function getSummary(Request $request, Response $response)
    {
        try {
            $filter = $this->crud->processFilter($request->getFilterCriteria());
            
            $params = array(
                "NMPRODUTO" => '%'.$filter['QUERY'].'%',
                "BRANCH_ID" => $this->sessionHandler->getSessionData('CDFILIAL'),
                "OPERATOR_ID" => $this->sessionHandler->getSessionData('CDOPERADOR')
            );
            
            $summary = $this->crud->executeQuery('search-summary', $params)[0];
            $data = $this->prepareDataSet($summary);
            
            $response->setMetadata($data);
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    /**
     * Prepares the data to send to the front end according to the 
     * standard informed in the requisition contract.
     * 
     * @param array $summary Data returned from the query that brings filter information
     *
     * @return array
     * 
     * @access private
     * @author Grazielle Ferreira Antunes 
     */
    private function prepareDataSet($summary) {
        
        $dataSet = array ();
        
        if($summary['SUPPLIERS_ID'] != null){
            
            $regexId = "/.+(?=_id)/i";
            $regexRange = "/(?<=^min|^max)_.+/i";
            $regexCleanRange = "/(?<=^min_|^max_).+/i";
    
            foreach($summary as $key => $value) {
                
                $matchesRange = array();
                $matchesCleanRange = array();
                preg_match($regexRange, $key, $matchesRange);
                preg_match($regexCleanRange, $key, $matchesCleanRange);
                $index = 0;
                
                if(count($matchesRange) == 0){
                    
                    $matches = array();
                    preg_match($regexId, $key, $matches);
                    
                    if(count($matches) > 0){
                        
                        $keyDataSet = $matches[0];
                        $results = array_unique($value);
                
                        foreach($results as $key => $value) {
                            
                            $dataSet[$keyDataSet][$index] = $dataSet[$keyDataSet][$index] ?? array("ID" => array(), "NAME" => array());
                            $dataSet[$keyDataSet][$index]['ID'] = $value;
                            $dataSet[$keyDataSet][$index]['NAME'] = $summary[$keyDataSet][$key];
                            $index++;
                        }
                    }
                } else {
                    
                    $propToSet = str_replace($matchesRange[0], "", $key);
                    $keyDataSet = $matchesCleanRange[0];
                    $dataSet[$keyDataSet] = $dataSet[$keyDataSet] ?? array("MIN" => array(), "MAX" => array());
                    $dataSet[$keyDataSet][$propToSet] = $value;
                }
        
            }
            $data = array("data" => $dataSet);
        } else {
            
            $data = array("data" => null);
        }
        return $data;
    }
	
}